#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
import os
import json
import ast
from ansible.module_utils.basic import AnsibleModule

try:
    from ..module_utils.rundb.basedb import BaseDB
    from ..module_utils.rundb.tinydb import TinyRunDB
    from ..module_utils.rundb.mongodb import MongoDB
except Exception:
    from linchpin.rundb.basedb import BaseDB
    from linchpin.rundb.tinydb import TinyRunDB
    from linchpin.rundb.mongodb import MongoDB

DB_DRIVERS = {
    "TinyRunDB": TinyRunDB,
    "MongoDB": MongoDB
}


def get_all_drivers():
    return DB_DRIVERS

# ---- Documentation Start ----------------#


DOCUMENTATION = """
---
version_added: "0.1"
module: rundb
short_description: A run database for linchpin, though it
                   could be used for anything transactional
                   really.
description:
  - This module allows a user to store and retrive values.
options:
  db_type:
    description:
      Type of Database used
    required: false
    default: TinyRunDB
  conn_str:
    description:
      Connection string to the database, can be file or url
    required: true
  db_schema:
    description:
      Database schema to use when running operations.
      The schema is required for an 'init' operation.
    required: true
  operation:
    description:
      Operation being performed on the database
      Available operations: init, update, purge, search
    required: true
    default: update
  run_id:
    description:
      The run_id value describes the current transaction.
      The run_id is returned for an 'init' operation.
      The run_id is required for an 'update' operation.
    required: false
    default: None
  table: (aka target in linchpin)
    description:
      Name of the table to update in the database
    required: true
  action:
    description:
      Action being performed, up/destroy
    required: false
    default: up
  key:
    description:
      key for the db record
    required: true (except for init and purge operations)
  value:
    description:
      value for the db record. Must be a dictionary of values.
    required: true (except for init and purge operations)

example


requirements: [TinyDB (or your own database)]
author: Clint Savage - herlo@redhat.com
"""


def main():


    module = AnsibleModule(
        argument_spec=dict(
            db_type=dict(type='str', required=False, default='TinyRunDB'),
            db_schema=dict(type='dict', required=True),
            conn_str=dict(type='str', required=True),
            operation=dict(choices=['init',
                                    'update',
                                    'purge',
                                    'get'],
                           default='update'),
            run_id=dict(type='int', required=False),
            table=dict(type='str', required=True),
            action=dict(type='str', required=False, default='up'),
            key=dict(type='str', required=False),
            value=dict(type='str', required=False),
        ),
    )
    db_type = module.params['db_type']
    db_schema = module.params['db_schema']
    conn_str = os.path.expanduser(module.params['conn_str'])
    op = module.params['operation']
    value = module.params['value']

    is_changed = False
    output = None
    try:
        rundb = BaseDB(DB_DRIVERS[db_type], conn_str=conn_str)
        rundb.schema = db_schema

        val = value
        # if value looks like a dict it is a dict!
        if value and (value.startswith("{") or value.startswith("[")):
            try:
                val = json.loads(value)
            except Exception:
                try:
                    val = ast.literal_eval(value)
                except Exception as e:
                    module.fail_json(msg=e)

        if op == "init":
            init_rundb(module, rundb, val)
        elif op == "purge":
            output = purge_rundb(rundb)
        elif op == "update":
            output = update_rundb(module, rundb, val)
        elif op == "get":
            output = get_item(module, rundb)
        else:
            msg = "Module 'action' required"
            module.fail_json(msg=msg)

        if output:
            is_changed = True

        module.exit_json(output=str(output), changed=is_changed)

    except Exception as e:
        module.fail_json(msg=str(e))


def update_rundb(module, rundb, value):
    table = module.params['table']
    run_id = module.params['run_id']
    key = module.params['key']

    # idempotent update
    if run_id and key and value:
        # get the record first
        runid = int(run_id)
        return rundb.update_record(table, runid, key, value)

    else:
        msg = ("'table', 'run_id, 'key', and 'value' required"
               " for update operation")
        module.fail_json(msg=msg)


def get_item(module, rundb):
    table = module.params['table']
    action = module.params['action']
    run_id = module.params['run_id']
    key = module.params['key']

    if key:
        runid = None
        if run_id:
            runid = int(run_id)
        if key == 'run_id':
            output = rundb.get_record(table,
                                      action=action,
                                      run_id=runid)[1]

        else:
            record = rundb.get_record(table,
                                      action=action,
                                      run_id=runid)[0]

            if key in record:
                output = record.get(key)
            else:
                msg = "key '{0}' was not found in"
                " record".format(key)
                module.fail_json(msg=msg)
        return output
    else:
        msg = "The 'key' value must be passed"
        module.fail_json(msg=msg)


def init_rundb(module, rundb, val):
    if val:
        rundb.schema = val
        return rundb.init_table(module.params['table'])
    else:
        msg = ("'table' and 'value' required for init operation")
        module.fail_json(msg=msg)


def purge_rundb(module, rundb):
    return rundb.purge(table=module.params['table'])


if __name__ == '__main__':
    main()



#!/usr/bin/env python


import linchpin.FilterUtils.FilterUtils as filter_utils


def filter_list_by_attr_val(output, attr, val):
    new_output = []
    for ele in output:
        if attr in ele:
            if ele[attr] == val:
                new_output.append(ele)
    return new_output


class FilterModule(object):
    ''' Filters a list by attr and value of each element '''
    def filters(self):
        return {
            'filter_list_by_attr_val': filter_utils.filter_list_by_attr_val
        }



from __future__ import absolute_import

import subprocess

import ansible.constants as ansible_constants


def install(package):
    cmd = "ansible-galaxy install {0}".format(package)
    retcode = subprocess.call(cmd,
                              shell=True)

    return retcode == 0


def get_role_paths():
    """
    Returns a list of all the roles paths which Ansible-galaxy uses when
    searching for roles

    This is wrapped in a function so that the logic can be expanded/modified in
    the future if need be
    """
    return ansible_constants.DEFAULT_ROLES_PATH



from __future__ import print_function
import sys
import json


prev_hooks = sys.argv[-2]
pipe_path = sys.argv[-1]
print("input: {0}".format(prev_hooks))

prev_hooks = json.loads(prev_hooks)
respipe = open(pipe_path, 'w')

# return successfully if the PinFile is in the list of files
for hook in prev_hooks:
    print(hook['data'])
    if type(hook['data']) is list and 'PinFile' in hook['data']:
        print('Found PinFile')
        sys.exit(0)

error = {'error': 'could not find \'PinFile\''}
print(error, file=respipe)
sys.exit(1)



#!/usr/bin/env python

from __future__ import absolute_import
import sys

from . import usedb
from .basedb import BaseDB

try:
    if sys.version_info[0] == 3:
        from pymongo import MongoClient, DESCENDING
except Exception:
    pass



class MongoDB(BaseDB):

    def __init__(self, conn_str):
        self.name = 'MongoDB'
        self.conn_str = conn_str
        self.default_table = 'linchpin'


    def _opendb(self):
        try:
            self.client = MongoClient(self.conn_str)
            self.db = self.client['linchpin']
        except Exception:
            pass

    def _closedb(self):
        # also flush the stored data to mongodb
        pass

    @property
    def schema(self):
        return self._schema


    @schema.setter
    def schema(self, schema):
        self._schema = dict()
        self._schema.update(schema)


    @usedb
    def init_table(self, table):
        table = self.db[table]
        prev_run = table.find_one(sort=[('_id', DESCENDING)])
        if prev_run:
            return prev_run['_id'] + 1
        return 1


    @usedb
    def update_record(self, table, run_id, key, value):
        collection = self.db[table]

        result = None
        if key == 'outputs' and 'resources' in value[0].keys():
            result = self._update_outputs(collection, run_id, key, value)
            if result:
                return result
        if type(self._schema[key]) == list:
            if isinstance(value, list):
                result = collection.find_one_and_update({'_id': run_id},
                                                        {'$addToSet':
                                                            {key: {'$each':
                                                                   value}}},
                                                        upsert=True)
            else:
                result = collection.find_one_and_update({'_id': run_id},
                                                        {'$addToSet': {key:
                                                                       value}},
                                                        upsert=True)
        else:
            result = collection.find_one_and_update({'_id': run_id},
                                                    {'$set': {key: value}},
                                                    upsert=True)

        # find_one_and_update() returns the document before the update
        # if the value is the same, we return None (to signify no change)
        # else we return the document
        if result and result.get(key, None) == value:
            return None
        return result


    def _update_outputs(self, collection, run_id, key, value):
        val = []
        for v in value:
            if type(v['resources']) == list:
                val.extend(v['resources'])
            else:
                val.append(v['resources'])
        return collection.find_one_and_update({'_id': run_id,
                                              'outputs.resources':
                                                  {'$exists': True}},
                                              {'$push': {"outputs.$.resources":
                                                         {'$each': val}}})



    @usedb
    def get_tx_record(self, table, tx_id):
        table = self.db[table]

        return table.find_one({'_id': tx_id})


    @usedb
    def get_tx_records(self, table, tx_ids):
        table = self.db[table]

        txs = {}
        for tx_id in tx_ids:
            txs[tx_id] = table.find_one({'_id': tx_id})

        return txs


    @usedb
    def get_run_id(self, table, action='up'):
        table = self.db[table]

        record = table.find_one({'action': action}, sort=[('_id', DESCENDING)])
        if record:
            return record['_id']


    @usedb
    def get_record(self, table, action=None, run_id=None):
        table = self.db[table]

        record = None

        if run_id:
            record = table.find_one({'_id': int(run_id)})
        if action:
            record = table.find_one({'action': action},
                                    sort=[('_id', DESCENDING)])
        else:
            record = table.find_one(sort=[('_id', DESCENDING)])

        if record:
            run_id = run_id if run_id else record['_id']
            return (record, run_id)
        return (None, 0)


    @usedb
    def get_records(self, table, count=10):
        table = self.db[table]

        if count == 'all':
            return table.find().sort(DESCENDING)
        return list(table.find(limit=count, sort=[('_id', DESCENDING)]))


    @usedb
    def get_tables(self):
        return self.db.list_collection_names()


    def remove_record(self, table, key, value):
        pass


    @usedb
    def search(self, table, key=None):
        table = self.db[table]

        if key:
            return table.find({key: {"$exists": True}})
        return table.find()


    def query(self, table, query):
        pass


    @usedb
    def purge(self, table):
        table = self.db[table]
        table.purge()



from __future__ import absolute_import
from tinydb import TinyDB
from tinydb.storages import JSONStorage
from tinydb.operations import add
from tinydb.operations import set as tinySet
from tinydb.middlewares import CachingMiddleware
from six.moves import range

from . import usedb
from .basedb import BaseDB

import tinydb


class TinyRunDB(BaseDB):

    def __init__(self, conn_str):
        self.name = 'TinyRunDB'
        self.conn_str = conn_str
        self.default_table_name = 'linchpin'


    def _opendb(self):
        self.middleware = CachingMiddleware(JSONStorage)
        self.middleware.WRITE_CACHE_SIZE = 4096
        tinydb_version = tinydb.version.__version__
        if int(tinydb_version.split(".")[0]) == 3:
            self.db = TinyDB(self.conn_str, storage=self.middleware,
                             default_table=self.default_table_name)
        else:
            self.db = TinyDB(self.conn_str, storage=self.middleware)
            self.db.default_table_name = self.default_table_name


    def __str__(self):
        if self.conn_str:
            return "{0} at {2}".format(self.name, self.conn_str)
        return "{0} at {1}".format(self.name, 'None')


    @property
    def schema(self):
        return self._schema


    @schema.setter
    def schema(self, schema):
        self._schema = dict()
        self._schema.update(schema)


    @usedb
    def init_table(self, table):
        t = self.db.table(name=table)
        return t.insert(self.schema)


    @usedb
    def update_record(self, table, run_id, key, value):
        t = self.db.table(name=table)
        # get transaction record
        tx_rec = t.get(doc_id=run_id).get("outputs", [])
        if len(tx_rec) > 0 and isinstance(value, list):
            # fetch the resources dict, index
            # by filtering them from outputs list
            res_list = [(idx, x) for idx, x in enumerate(tx_rec)
                        if "resources" in x]
            if len(res_list) != 0:
                res_idx = res_list[0][0]
                resources = res_list[0][1]
                if "resources" in list(value[0].keys()):
                    de = resources["resources"]
                    for i in value[0]["resources"]:
                        de.append(i)
                    de = {"resources": de}
                    tx_rec[res_idx] = de
                    res = t.update(tinySet(key, [de]), doc_ids=[run_id])
                    return res
        changed = t.update(add(key, value), doc_ids=[run_id])
        return changed


    @usedb
    def get_tx_record(self, table, tx_id):

        t = self.db.table(name=table)
        return t.get(doc_id=tx_id)


    @usedb
    def get_tx_records(self, table, tx_ids):

        txs = {}
        t = self.db.table(name=table)
        for tx_id in tx_ids:
            txs[tx_id] = t.get(doc_id=tx_id)

        return txs


    @usedb
    def get_run_id(self, table, action='up'):
        """
        gets the run_id associated with the most recent instance of `action`
        if there is no instance of action, returns the most recent run_id
        """
        t = self.db.table(name=table)
        run_id = len(t.all())

        for rid in range(int(run_id), 0, -1):
            record = t.get(doc_id=int(rid))
            if record and record['action'] == action:
                return rid
        return run_id


    @usedb
    def get_record(self, table, action=None, run_id=None):

        t = self.db.table(name=table)
        if run_id:
            record = t.get(doc_id=int(run_id))
            if record:
                return(record, int(run_id))
        if action:
            run_id = self.get_run_id(table, action)
            if not run_id:
                return (None, 0)
            record = t.get(doc_id=int(run_id))
            if record and record['action'] == action:
                return (record, int(run_id))
        else:
            record = t.get(doc_id=len(t))
            run_id = record['run_id']

        if record:
            return (record, int(run_id))
        return (None, 0)


    @usedb
    def get_records(self, table, count=10):
        records = {}
        if table in self.db.tables():
            t = self.db.table(name=table)
            if len(t.all()):
                start = len(t)
                if count == 'all':
                    end = 0
                else:
                    end = start - count
                for i in range(start, end, -1):
                    records[i] = t.get(doc_id=i)
        return records


    @usedb
    def get_tables(self):

        tables = self.db.tables()
        tables.remove(self.default_table)

        return tables


    def remove_record(self, table, key, value):
        pass


    def search(self, table, key=None):
        t = self.db.table(name=table)
        if key:
            return t.search(key)
        return t.all()


    def query(self, table, query):
        pass


    def purge(self, table=None):
        if table:
            return self.db.purge_table(table)
        return self.db.purge_tables()


    def _closedb(self):
        self.db.close()



#!/usr/bin/env python

from __future__ import absolute_import

import ast
import io
import os

from setuptools import setup, find_packages

with open('linchpin/version.py') as f:
    for line in f:
        if line.startswith('__version__'):
            ver = ast.parse(line).body[0].value.s
            break

dir_path = os.path.dirname(os.path.realpath(__file__))

# reading description from README.rst
with io.open(os.path.join(dir_path, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

# reading requirements from requirements.txt
with io.open(os.path.join(dir_path, 'requirements.txt')) as f:
    required = f.read().splitlines()

install_required = list(required)

ignore_dir = ['.git']

azure_deps = open("requirements-azure.txt", "r").readlines()

setup(
    name='linchpin',
    version=ver,
    description='Ansible-based multi-cloud provisioner',
    long_description=long_description,
    author='samvaran kashyap rallabandi',
    author_email='linchpin@redhat.com',
    url='http://linchpin.readthedocs.io/',
    install_requires=install_required,
    entry_points='''
        [console_scripts]
        linchpin=linchpin.shell:runcli
    ''',
    tests_require=["pytest<=4.4.0",
                   "nose",
                   "mock",
                   "coverage",
                   "flake8",
                   "molecule<=2.22",
                   "radon"],
    extras_require={
        'krbV': ["python-krbV"],
        'beaker': ['beaker-client>=27.0', 'bottle', 'gssapi'],
        'docs': ["docutils", "sphinx", "sphinx_rtd_theme", "sphinx-automodapi"],
        'tests': ["nose", "mock", "coverage", "flake8",
                  "pytest<=4.4.0", "pytest-runner", "molecule<=2.22",
                  "molecule[docker]", "radon"],
        'libvirt': ["libvirt-python>=3.0.0", "lxml"],
        'vmware': ["PyVmomi>=6.7.1"],
        'docker': ["docker-py>=1.7.0"],
        'azure': azure_deps,
        'openshift': ['openshift>=0.8.6,<0.8.8', 'kubernetes-validate']
    },
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    scripts=[
        'scripts/install_libvirt_deps.sh',
        'scripts/install_selinux_venv.sh'
    ]
)



#!/usr/bin/env python

from __future__ import absolute_import
from collections import OrderedDict

from linchpin.InventoryFilters.InventoryFilter import InventoryFilter


class Inventory(InventoryFilter):
    DEFAULT_HOSTNAMES = ['public_ip']

    def get_host_data(self, res, cfgs):
        """
        Returns a dict of hostnames or IP addresses for use in an Ansible
        inventory file, based on available data. Only a single hostname or IP
        address will be returned per instance, so as to avoid duplicate runs of
        Ansible on the same host via the generated inventory file.

        Each hostname contains mappings of any variable that was defined in the
        cfgs section of the PinFile (e.g. __IP__) to the value in the field that
        corresponds with that variable in the cfgs.

        By default, the hostname will be the public_ip field returned by gcloud

        :param topo:
            linchpin GCloud resource data

        :param cfgs:
            map of config options from PinFile
        """
        if res['resource_group'] != 'gcloud':
            return OrderedDict()

        if res['role'] == 'gcloud_gce':
            return self.get_gcloud_gce_host_data(res, cfgs)
        else:
            return OrderedDict()

    def get_gcloud_gce_host_data(self, res, cfgs):
        host_data = OrderedDict()

        var_data = cfgs.get('gcloud', {})
        if var_data is None:
            var_data = {}
        for instance in res['instance_data']:
            host = self.get_hostname(instance, var_data,
                                     self.DEFAULT_HOSTNAMES)
            hostname_var = host[0]
            hostname = host[1]
            host_data[hostname] = {}
            if '__IP__' not in list(var_data.keys()):
                var_data['__IP__'] = hostname_var
                host_data[hostname] = {}
            self.set_config_values(host_data[hostname], instance, var_data)
        return host_data



from nose.tools import with_setup
from nose.tools import assert_true
from nose.tools import assert_equal
from nose.tools import assert_dict_equal

from linchpin.hooks.action_managers import RubyActionManager


# ------------------------------- #
#    Setup functions for tests    #
# ------------------------------- #
def setup_ruby_action_manager():
    global manager

    name = "ruby"
    action_data = {'name': 'success_hook',
                   'context': True,
                   'path': 'linchpin/tests/mockdata/dummy/hooks/ruby/success_hook',
                   'actions': ['success_hook.rb']
    }
    target_data = {'inventory_dir': 'inventories',
                   'inventory_file': 'docs/source/examples/workspaces/dummy-hook-flags/inventories/dummy-da1711.inventory'}
    context = True
    use_shell = True
    state = "postup"
    kwargs = {'context': True, 'verbosity': 2, 'use_shell': False}

    manager = RubyActionManager(name, action_data, target_data, state=state, context=context, use_shell=use_shell)


# ----------- #
#    Tests    #
# ----------- #
@with_setup(setup_ruby_action_manager)
def test_validate():
    status = manager.validate()
    # the status will be 'True' if the validation succeeded
    assert_true(status)


@with_setup(setup_ruby_action_manager)
def test_load():
    pass


@with_setup(setup_ruby_action_manager)
def test_add_ctx_params():
    # test add_ctx_params when context=False
    hook_path = '/tmp/hook_path'
    results = 'results'
    data_path = '/tmp/data_path'
    params = manager.add_ctx_params(hook_path, results, data_path, False)
    expected_params = "{0} -- '{1}' {2}".format(hook_path, results, data_path)
    assert_equal(params, expected_params)
    # test add_ctx_params when context=True
    params = manager.add_ctx_params(hook_path, results, data_path, True)
    expected_params = "{0} inventory_dir=inventories inventory_file=docs/source/examples/workspaces/dummy-hook-flags/inventories/dummy-da1711.inventory -- '{1}' {2}".format(hook_path, results, data_path)
    print(expected_params)
    print(params)
    assert_equal(params, expected_params)



@with_setup(setup_ruby_action_manager)
def test_execute():
    expected_results = {'state': 'postup', 'return_code': 0, 'data': ''}
    results = manager.execute([])
    assert_equal(len(results), 1)
    assert_dict_equal(results[0], expected_results)



# """
# example Pinfile for reference::
# ---
# openstack:
#   topology: ex_os_server.yml
#   layout: openshift-3node-cluster.yml
#   hooks:
#     postup:              # sub-state is specified
#         - name: do_something
#           type: shell
#           actions:
#             - samvaran
#         - name: manipulate_inventory
#           type: shell
#           path: /tmp/shellscripts
#           actions:
#             - thisisshell.sh
#         - name: post_up
#           type: ansible
#           actions:
#             - playbook: test_playbook.yaml
#               vars: test_var.yaml
#               extra_vars: { 'testvar': 'world'}
#     preup:              # sub-state is specified
#         - name: do_something
#           type: shell
#           actions:
#             - echo ' this is post up operation Hello hai how r u ?'
#         - name: build_openshift_cluster
#           type: ansible
#           actions:
#             - playbook: test_playbook.yaml
#               vars: test_var.yaml
#               extra_vars: { 'testvar': 'world'}
#     postdestroy:
#         - name: do_something
#           type: shell
#           actions:
#             - echo ' this is post up operation Hello hai how r u ?'
#         - name: postdown_task
#           type: ansible
#           actions:
#             - playbook: test_playbook.yaml
#               vars: test_var.yaml
#               extra_vars: { 'testvar': 'world'}
#
# """
from __future__ import absolute_import
import os
import ast
import sys
import git
from git.exc import GitCommandError
import glob
import shutil

from linchpin.hooks.action_managers import ACTION_MANAGERS
from linchpin.hooks.built_ins import GLOBAL_HOOKS
from linchpin.exceptions import ActionManagerError
from linchpin.exceptions import HookError



class ActionBlockRouter(object):
    """
    Proxy pattern implementation for fetching actionmanagers by name
    """

    def __init__(self, name, *args, **kwargs):

        self.__implementation = self.__get_implementation(name)(name,
                                                                *args,
                                                                **kwargs)

    def __getattr__(self, name):

        return getattr(self.__implementation, name)

    def __get_implementation(self, class_name):
        '''
        Fetches implementation of lcass from linchpin.hooks.action_managers
        :param name: action manager class name
        '''

        action_class = ACTION_MANAGERS.get(class_name, None)
        if action_class is None:
            raise ActionManagerError('Action Class {0}'
                                     'not found '.format(class_name))
        return action_class


class LinchpinHooks(object):


    def __init__(self, api):
        """
        LinchpinHooks class constructor
        :param api: LinchpinAPI object
        """

        self.api = api
        self.api.bind_to_hook_state(self.run_hooks)
        self._rundb = None
        self._rundb_id = None
        self._prev_rundb_id = None
        self.use_shell = self.api.get_cfg("ansible", "use_shell")
        self.verbosity = self.api.ctx.verbosity


    @property
    def rundb(self):
        return (self._rundb, self._rundb_id)


    @rundb.setter
    def rundb(self, data):
        self._rundb = data[0]
        self._rundb_id = data[1]
        # Only set this if we are running destroy hooks
        # _prev_rundb_id is the id of the corresponding "up"
        if len(data) == 3:
            self._prev_rundb_id = data[2]


    def prepare_ctx_params(self):
        """
        prepares few context parameters based on the current target_data
        that is being set. these parameters are based topology name.
        """

        topo_data = self.api.get_evar('topo_data', {})

        workspace = self.api.get_evar('workspace')
        inv_folder = self.api.get_evar('inventories_folder')
        topo_name = topo_data.get('topology_name')
        uhash = self.api.get_evar('uhash')
        uhash_enabled = self.api.get_evar('enable_uhash', False)
        ext = self.api.get_cfg('extensions', 'inventory')
        # some applications need no_monitor disabled for ansible hooks
        no_monitor = self.api.get_evar('no_monitor')

        inv_file = '{0}/{1}/{2}{3}'.format(workspace,
                                           inv_folder,
                                           topo_name,
                                           ext)
        if uhash_enabled:
            inv_file = '{0}/{1}/{2}-{3}{4}'.format(workspace,
                                                   inv_folder,
                                                   topo_name,
                                                   uhash,
                                                   ext)

        self.api.target_data['extra_vars'] = {}
        self.api.target_data['extra_vars']['inventory_dir'] = inv_folder
        self.api.target_data['extra_vars']['inventory_file'] = inv_file
        self.api.target_data['extra_vars']['no_monitor'] = no_monitor


    def prepare_inv_params(self):

        target = self.api.get_evar('target', default=None)
        action = self.api.get_evar('_action', default='up')

        topology_data = {}
        layout_data = {}
        results_data = {}

        data, run_id = self._rundb.get_record(target,
                                              action=action,
                                              run_id=self._rundb_id)

        inputs = [i for i in data['inputs'] if data['inputs']]
        outputs = [i for i in data['outputs'] if data['outputs']]

        for inp in inputs:
            if 'layout_data' in inp:
                layout_data = inp
            if 'topology_data' in inp:
                topology_data = inp

        for out in outputs:
            if 'resources' in out:
                results_data = out

        return (topology_data, layout_data, results_data)


    def run_hooks(self, state, is_global=False):
        """
        Function to run hook all hooks from Pinfile based on the state
        :param state: hook state (currently, preup, postup,
        predestroy, postdestroy)
        :param is_global: whether the hook is global (can be applied to
        multiple targets)
        """
        hooks_data = self.api.get_evar('hooks_data', None)

        self.prepare_ctx_params()

        # this will replace the above target_data and pull from the rundb
        # run_data = self.prepare_inv_params()
        if str(state) == 'postinv':
            run_data = self.prepare_inv_params()
            return self.run_inventory_gen(run_data)

        if hooks_data and str(state) in hooks_data:
            self.api.ctx.log_debug('running {0} hooks'.format(state))

            # fetches all the state_data , ie., all the action blocks inside
            # state of the target

            state_data = hooks_data.get(str(state), None)

            # Print out error message if the hooks are not found
            if state_data is None:
                self.api.ctx.log_state('{0} hook not found'
                                       ' in PinFile'.format(state))
                return

            # current target data extravars are fetched
            tgt_data = self.api.target_data.get('extra_vars', None)
            self.run_actions(state, state_data, tgt_data)


    def run_inventory_gen(self, data):
        # determine provisioned resources

        # map outputted resources to inventory layout

        # save to file
        pass


    def run_actions(self, state, action_blocks, tgt_data, is_global=False):
        """
        Runs actions inside each action block of each target

        :param action_blocks: list of action_blocks each block constitues
                       to a type of hook
        :param tgt_data: data specific to target, which can be dict of
        topology , layout, outputs, inventory
        :param is_global: scope of the hook

        example: action_block:
        - name: do_something
          type: shell
          actions:
            - echo ' this is 'postup' operation Hello hai how r u ?'
        """


        if is_global:
            raise NotImplementedError('Run Hooks is not implemented \
                                       for global scoped hooks')
        else:
            self.run_local_actions(state, action_blocks, tgt_data)


    def run_local_actions(self, state, action_blocks, tgt_data):
        # a_b -> abbr for action_block
        for a_b in action_blocks:
            self.run_action(state, a_b, tgt_data)


    def run_action(self, state, block, tgt_data):
        use_shell = ast.literal_eval(self.api.get_cfg("ansible", "use_shell"))
        if block['name'] in GLOBAL_HOOKS.keys():
            block = self.global_hooks_block(block)
        # currently built-ins support only ansible
        action_type = block.get('type', 'ansible')
        ab_ctx = block['context'] if 'context' in block else False
        if 'path' not in block:
            self.resolve_block_path(block)

        if 'action_manager' in block:
            class_ = self.get_custom_action_manager_class(block)
            block_obj = class_(action_type,
                               block,
                               tgt_data,
                               context=ab_ctx,
                               state=state,
                               verbosity=self.verbosity,
                               use_shell=use_shell)
        else:
            block_obj = ActionBlockRouter(action_type,
                                          block,
                                          tgt_data,
                                          context=ab_ctx,
                                          state=state,
                                          verbosity=self.verbosity,
                                          use_shell=use_shell)
        try:
            self.api.ctx.log_state('-------\n'
                                   'start hook'
                                   ' {0}:{1}'.format(block['type'],
                                                     block['name']))


            # validates the class object
            block_obj.validate()

            if 'src' in block.keys():
                self.fetch_src(block)

            target = self.api.get_evar('target', default=None)
            hook_result = self.execute_hook(block_obj, target)
            # write results to rundb
            self._rundb.update_record(target,
                                      self._rundb_id,
                                      'hooks',
                                      hook_result)
            for result in hook_result:
                if result['return_code'] > 0:
                    raise HookError("Hook execution failed")

            # intentionally using print here
            self.api.ctx.log_state('end hook {0}:{1}\n-------'.format(
                                   block['type'], block['name']))

        except Exception as e:
            dflt = self.api.get_cfg("hook_flags",
                                    "ignore_failed_hooks")
            if not dflt:
                raise HookError("Error executing hook: '{0}'".format(e))
            self.api.ctx.log_info(str(e))

    def get_custom_action_manager(self, action_block):
        # fetches the action object from the path
        # add path to python path
        sys.path.append(action_block['path'])
        # get the module path
        module_path = '{0}{1}'.format(action_block['path'],
                                      action_block['action_manager'])
        if os.path.exists(action_block['action_manager']):
            module_path = action_block['action_manager']
        # get module src
        module_src = open(module_path, 'r').read()
        # strip .py ext from module path
        module_path = module_path.strip('.py')
        # strip .py ext from action_manager
        action_block['action_manager'] = action_block['action_manager'] \
            .strip('.py')
        # parse the module
        module_src = ast.parse(module_src)
        # get all classes inside the class
        classes = ([node.name
                    for node in ast.walk(module_src)
                    if isinstance(node, ast.ClassDef)])

        # choose the first name as the class name
        class_name = classes[0]
        # import the module with class name, it should work
        module = __import__(action_block['action_manager'])
        # get the class and return it
        return getattr(module, class_name)
        # action_block_obj is action_block_object


    def execute_hook(self, block_obj, target):
        # produce a partial application of update_record to limit
        # data passed to action managers
        # get past results from rundb
        action = 'hooks'
        record = self._rundb.get_record(target,
                                        action,
                                        self._rundb_id)

        # if this is set, then we're running a destroy hook of some kind
        if self._prev_rundb_id:
            prev_record = self._rundb.get_record(target,
                                                 action,
                                                 self._prev_rundb_id)
            # prepend the data from the `linchpin up` phase so
            # that the hooks are read in their run order
            record = prev_record + record
        # executes the hook
        hook_record = dict(record[0])['hooks']
        return block_obj.execute(hook_record)


    def fetch_src(self, block):
        # fetch the src
        src_type = block['src']['type']
        # we can add other source types later
        # e.g. 'ftp', 'svn', or 'local' for hooks that exist ib
        # a different location on the local machine
        if src_type == 'git':
            self.fetch_git_src(block)
        else:
            raise HookError("Invalid type for src")

    def fetch_git_src(self, block):
        git_remote = block['src']['url']
        # clone repository
        cwd = os.getcwd()
        try:
            git.Git(cwd).clone(git_remote)
        except GitCommandError as e:
            err_str = "already exists and is not an" + \
                " empty directory"
            if err_str not in e.stderr:
                raise
            pass
        # move files to relevant section of hooks dir
        dest_dir = "{0}/hooks/{1}/{2}".format(cwd,
                                              block['type'],
                                              block['name'])
        try:
            os.makedirs(dest_dir)
        except OSError as e:
            print(e)
        repo_name = git_remote.rsplit('/', 1)[-1]
        for filename in glob.glob(os.path.join(cwd,
                                               repo_name,
                                               block['type'],
                                               block['name'],
                                               '*.*')):
            if os.path.isdir(filename):
                shutil.copytree(filename, dest_dir)
            else:
                shutil.copy(filename, dest_dir)
        # remove old directory
        shutil.rmtree(os.path.join(cwd, repo_name))

    def resolve_block_path(self, block):
        # modify block['path']
        # dicts in python are mutable so it doesn't need to
        # be passed back

        # search in global_hooks first if found resolve to
        # global_hooks_path
        if block['name'] in GLOBAL_HOOKS.keys():
            # cp ~ current_file_path
            cfp = os.path.realpath(__file__).split("/")[0:-1]
            cfp = "/".join(cfp)
            block["path"] = '{0}/built_ins/{1}/'.format(
                cfp, block["name"])
        # if the path is not defined it defaults to
        # workspace/hooks/typeofhook/name
        else:
            block['path'] = '{0}/{1}/{2}/{3}/'.format(
                self.api.ctx.workspace,
                self.api.get_evar('hooks_folder',
                                  default='hooks'),
                block.get('type', 'ansible'),
                block['name'])

    def global_hooks_block(self, block):
        t_block = GLOBAL_HOOKS.get(block['name'])
        # update the extra_vars dict
        # fetch vars from linchpin.conf
        # and update them as context vars
        t_block["actions"][0]["extra_vars"].update(
            self.api.get_evar())
        t_block["actions"][0]["extra_vars"].update(
            block.get("extra_vars", {}))
        return t_block



# This directory contains all of the unit tests for linchpin.  The file
# tree within here is idential to that of the linchpin package.  Each
# file has a corresponding test_<filename>_pass and test_<filename>_fail
# file.  So linchpin/cli/__init__.py is tested via tests/cli/test_init_pass.py
# and tests/cli/test_init_fail.py.  As the name implies, *_fail.py tests all
# cases that should fail.  This includes cases in which the function returns
# a return code that is greater than 0 or throws an error.  *_pass.py should
# include all other test cases.
#
# Within each file, any given function should have one or more test functions
# beginning with test_<function_name>.  So the _write_to_inventory() function
# may have a test named test_write_to_inventory().  The do_action() function may
# have a test case named test_do_action_up() and another named
# test_do_action_destroy().
#
# When writing tests, aim for 100% coverage, but also make sure that the tests
# are thorough.  If you run a function and don't check that it returns the
# value you expect, that code isn't being properly tested regardless of
# coverage



from __future__ import absolute_import
from collections import OrderedDict

from linchpin.InventoryFilters.InventoryFilter import InventoryFilter


class Inventory(InventoryFilter):
    DEFAULT_HOSTNAMES = ['Config.Hostname']

    def get_host_data(self, res, config):
        # Only docker_container resource type produces host data.
        if res['resource_group'] != 'docker':
            return {}

        if res['role'] == 'docker_container':
            return self.get_docker_container_host_data(res, config)
        else:
            return {}

    def get_docker_container_host_data(self, res, config):
        host_data = OrderedDict()

        var_data = config.get('docker', {})
        if var_data is None:
            var_data = {}
        host = self.get_hostname(res, var_data,
                                 self.DEFAULT_HOSTNAMES)
        hostname_var = host[0]
        hostname = host[1]
        host_data[hostname] = {}
        if '__IP__' not in var_data.keys():
            var_data['__IP__'] = hostname_var
        self.set_config_values(host_data[hostname], res, var_data)
        return host_data



from nose.tools import with_setup
from nose.tools import assert_true
from nose.tools import assert_equal
from nose.tools import assert_dict_equal

from linchpin.hooks.action_managers import AnsibleActionManager


# ------------------------------- #
#    Setup functions for tests    #
# ------------------------------- #
def setup_ansible_action_manager():
    global manager

    name = "ansible"
    action_data = {'name': 'success_hook',
                   'context': True,
                    'path': 'docs/source/examples/workspaces/dummy-hook-flags/hooks/ansible/success_hook/',
                   'actions': [{
                       'playbook': 'success_hook.yaml'
                   }]}
    target_data = {'inventory_dir': 'inventories',
                   'inventory_file': 'docs/source/examples/workspaces/dummy-hook-flags/inventories/dummy-da1711.inventory'}
    context = True
    use_shell = True
    state = "postup"
    kwargs = {'context': True, 'verbosity': 2, 'use_shell': False}

    manager = AnsibleActionManager(name, action_data, target_data, state=state, context=context, use_shell=use_shell)


# ----------- #
#    Tests    #
# ----------- #
@with_setup(setup_ansible_action_manager)
def test_validate():
    status = manager.validate()
    # the status will be 'True' if the validation succeeded
    assert_true(status)


@with_setup(setup_ansible_action_manager)
def test_load():
    pass


@with_setup(setup_ansible_action_manager)
def test_get_ctx_params():
    params = manager.get_ctx_params()
    assert_equal(list(params.keys()), ['resource_file',
                                     'layout_file',
                                     'inventory_file',
                                       'no_monitor'])

@with_setup(setup_ansible_action_manager)
def test_execute():
    expected_results = {'state': 'postup', 'return_code': 0, 'data': []}
    results = manager.execute([])
    assert_equal(len(results), 1)
    assert_dict_equal(results[0], expected_results)



from __future__ import absolute_import
from __future__ import print_function
import os
import sys

from nose.tools import assert_equal
from nose.tools import assert_dict_equal
from nose.tools import assert_is_instance
from nose.tools import assert_dict_contains_subset
from nose.tools import with_setup

from linchpin import LinchpinAPI
from linchpin.utils.dataparser import DataParser
from linchpin.context import LinchpinContext
from linchpin.rundb import RunDB

from linchpin.tests.mockdata.contextdata import ContextData
import six

import json
import linchpin
from linchpin.api import Pinfile
from linchpin.api import Workspace


def test_api_create():

    lpc = LinchpinContext()
    lpc.load_config()
    lpc.load_global_evars()
    lpa = LinchpinAPI(lpc)

    assert_equal(isinstance(lpa, LinchpinAPI), True)


def setup_load_config():

    """
    Perform setup of ContextData() object, and run get_temp_filename()
    """

    global cd
    global provider
    global config_path
    global config_data

    provider = 'dummy'

    cd = ContextData()
    cd.load_config_data()
    cd.create_config()
    config_path = cd.get_temp_filename()
    config_data = cd.cfg_data
    cd.write_config_file(config_path)


def setup_lp_api():

    """
    Perform setup of LinchpinContext, lpc.load_config, and LinchPinAPI
    """

    global lpc
    global lpa
    global pf_data
    global target
    global provision_data

    setup_load_config()

    lpc = LinchpinContext()
    lpc.load_config(search_path=[config_path])
    lpc.load_global_evars()
    lpc.setup_logging()

    lpa = LinchpinAPI(lpc)

    pinfile = lpc.get_cfg('init', 'pinfile', default='PinFile')

    base_path = '{0}'.format(os.path.dirname(
        os.path.realpath(__file__))).rstrip('/')
    mock_path = '{0}/{1}/{2}'.format(base_path, 'mockdata', provider)

    if not lpa.workspace:
        lpa.workspace = mock_path

    lpa.set_evar('workspace', mock_path)

    pf_w_path = '{0}/PinFile'.format(mock_path, pinfile)

    parser = DataParser()
    pf_d = None
    pf_data = parser.process(pf_w_path)

    topo_folder = lpc.get_evar('topologies_folder')
    topo_file = pf_data[provider]["topology"]
    topo_path = '{0}/{1}/{2}'.format(mock_path, topo_folder, topo_file)
    with open(topo_path, 'r') as topo_stream:
        topology_data = parser.parse_json_yaml(topo_stream)
        provision_data = {provider: {'topology': topology_data}}


def setup_api():
    """
    performs initialization required for linchpin api
    """

    global wksp
    global pfile
    base_path = '{0}'.format(os.path.dirname(os.path.realpath(__file__)))\
        .rstrip('/')
    wksp = Workspace(path=base_path+"/mockdata/api_workspace")
    pinfile = open(base_path+"/mockdata/api_workspace/PinFile","r").read()
    pinfile = json.loads(pinfile)
    pfile = Pinfile(pinfile=pinfile)



@with_setup(setup_lp_api)
def test_setup_rundb():

    rundb = lpa.setup_rundb()
    assert_is_instance(rundb, RunDB)


@with_setup(setup_lp_api)
def test_get_cfg_section():

    lpa.set_cfg('test', 'key', 'value')
    lpa_dict = lpa.get_cfg('test')

    assert_dict_equal(lpa_dict, lpa.ctx.cfgs.get('test'))


@with_setup(setup_lp_api)
def test_get_cfg_item():

    lpa.set_cfg('test', 'key', 'value')
    lpa_value = lpa.get_cfg('test', 'key')

    test_dict = lpa.ctx.cfgs.get('test')
    test_value = test_dict.get('key')

    assert_equal(lpa_value, test_value)


@with_setup(setup_lp_api)
def test_set_cfg():

    test_dict = {'key': 'value', 'key2': 'value2'}

    for k, v in test_dict.items():
        lpa.set_cfg('test', k, v)

    assert_dict_equal(test_dict, lpa.ctx.cfgs.get('test'))


@with_setup(setup_lp_api)
def test_get_all_evars():

    test_evars = {'ekey1': 'evalue1', 'ekey2': 'evalue2'}
    for k, v in test_evars.items():
        lpa.set_evar(k, v)

    lpa_evars = lpa.get_evar()

    assert_dict_contains_subset(test_evars, lpa_evars)


@with_setup(setup_lp_api)
def test_get_evar_item():

    test_evars = {'ekey2': 'evalue2'}
    for k, v in test_evars.items():
        lpa.set_evar(k, v)

    assert_equal(lpa.get_evar(key='ekey2'), test_evars.get('ekey2'))


@with_setup(setup_lp_api)
def test_set_evar():

    test_evars = {'ekey': 'evalue'}

    for k, v in test_evars.items():
        lpa.set_evar(k, v)
    lpa_evars = lpa.get_cfg('evars')

    assert_dict_contains_subset(test_evars, lpa_evars)


@with_setup(setup_lp_api)
def test_set_hook_state():

    pass


@with_setup(setup_lp_api)
def test_bind_to_hook_state():

    pass



@with_setup(setup_lp_api)
def test_do_action():

    return_code, results = lpa.do_action(provision_data)

    failed = False
    if return_code:
        failed = True
        for target, data in six.iteritems(results):
            task_results = data['task_results'][0]

            if not isinstance(task_results, int):
                trs = task_results
                if trs is not None:
                    trs.reverse()
                    tr = trs[0]
                    if tr.is_failed():
                        msg = tr._check_key('msg')
                        print(("Target '{0}': {1} failed with"
                              " error '{2}'".format(target,
                                                    tr._task,
                                                    msg)))
            else:
                if task_results:
                    return_code = task_results

    assert not failed

@with_setup(setup_lp_api)
def test_do_validation():
    return_code, results = lpa.do_validation(provision_data)

    failed = False
    if return_code:
        failed = True
        for target, data in six.iteritems(results):
            if not data.startswith("valid"):
                print(("Validation for target '{0}': has failed with"
                      " error '{1}'".format(target, data)))

    assert not failed

@with_setup(setup_lp_api)
def test_lp_journal():

    x = lpa.lp_journal(targets=[provider])
    assert x.get(provider)


@with_setup(setup_lp_api)
def test_invoke_playbooks():

    topo = provision_data.get(provider).get('topology')
    resources = topo.get('resource_groups')

    rundb = lpa.setup_rundb()  # noqa F841
    lpa.set_evar('rundb_id', 1)

    lpa.set_evar('topo_data', topo)
    lpa.set_evar('target', provider)
    lpa.set_evar('resources', resources)
    lpa.set_evar('uhash', 'test')
    lpa.set_evar('use_uhash', True)
    lpa.set_evar('rundb_schema', {'action': '',
                                  'inputs': [],
                                  'outputs': [],
                                  'start': '',
                                  'end': '',
                                  'rc': 0,
                                  'uhash': ''})

    return_code, results = lpa._invoke_playbooks(resources,
                                                 action='up',
                                                 console=True)

    assert return_code == 0

"""

class Workspace:

    def set_workspace(self, path):

        self.workspace_path = path
        self.context.set_cfg('lp', 'workspace', self.workspace_path )
        self.context.set_evar('workspace', self.workspace_path)
        return self.workspace_path

    def set_evar(self, key, value):

        self.context.set_evar(key, value)
        return key,value

    def set_credentials_path(self, creds_path):
        if os.path.isdir(creds_path):
            return self.context.set_evar("default_credentials_path",
                                         creds_path)
        raise LinchpinError("Incorrect file path, path should be a directory")

    def set_vault_encryption(self, vault_enc):
        if isinstance(vault_enc, bool):
            return self.context.set_evar("vault_password",vault_pass)
        raise LinchpinError("Incorrect datatype please use boolean")

    def set_no_hooks(self, flag):
        if isinstance(flag, bool):
            return self.ctx.set_cfg("hookflags", "no_hooks", flag)
        raise LinchpinError("Incorrect datatype please use boolean")

    def set_ignore_failed_hooks(self, flag):
        if isinstance(flag, bool):
            return self.ctx.set_cfg("hookflags", "ignore_failed_hooks", flag)
        raise LinchpinError("Incorrect datatype please use boolean")

    def set_vault_pass(self, vault_pass):
        return self.context.set_evar("vault_password",vault_pass)

"""
@with_setup(setup_api)
def test_api_workspace():
    assert_equal(isinstance(wksp, Workspace), True)

@with_setup(setup_api)
def test_api_pinfile():
    assert_equal(isinstance(pfile, Pinfile), True)

@with_setup(setup_api)
def test_api_wksp_load_data():
    pindict = wksp.load_data(wksp.find_pinfile())
    expected = json.loads(open(wksp.find_pinfile(), "r").read())
    assert_equal(expected, pindict)

@with_setup(setup_api)
def test_api_wksp_validate():
    out = wksp.validate()
    expected = (0, {'dummy-test': {'layout': 'valid', 'topology': 'valid'}})
    assert_equal(expected, out)
    pass

@with_setup(setup_api)
def test_api_wksp_find_pinfile():
    assert_equal(os.path.isfile(wksp.find_pinfile()), True)
    pass

@with_setup(setup_api)
def test_api_wksp_set_workspace():
    out = wksp.set_workspace("/tmp/")
    assert_equal( out, "/tmp/")

@with_setup(setup_api)
def test_api_wksp_set_evar():
    wksp.set_evar("test", "var")
    assert_equal(wksp.get_evar("test"), "var")

@with_setup(setup_api)
def test_api_wksp_set_credentials_path():
    wksp.set_credentials_path("/tmp/")
    assert_equal(wksp.get_credentials_path(), "/tmp/")

@with_setup(setup_api)
def test_api_wksp_set_vault_encryption():
    wksp.set_vault_encryption(True)
    assert_equal(wksp.get_vault_encryption(), True)

@with_setup(setup_api)
def test_api_wksp_set_flag_no_hooks():
    wksp.set_flag_no_hooks(True)
    assert_equal(wksp.get_flag_no_hooks(), True)

@with_setup(setup_api)
def test_api_wksp_set_flag_ignore_failed_hooks():
    wksp.set_flag_ignore_failed_hooks(True)
    assert_equal(wksp.get_flag_ignore_failed_hooks(), True)

@with_setup(setup_api)
def test_api_wksp_set_vault_pass():
    wksp.set_vault_pass("hello")
    assert_equal(wksp.get_vault_pass(), "hello")

@with_setup(setup_api)
def test_api_wksp_get_inventory():
    wksp.up()
    inven = wksp.get_inventory()
    assert_equal.__self__.maxDiff = None
    expected = {u'dummy-test': '{"test": {"hosts": ["web-0"]}, "all": {"hosts": ["web-0", "web-1", "web-2", "test"], "vars": {"ansible_user": "root"}}, "example": {"hosts": ["web-1", "web-2", "test"]}, "_meta": {"hostvars": {"test": "test", "web-0": "web-0", "web-1": "web-1", "web-2": "web-2"}}}'}
    expected = json.loads(expected['dummy-test']).get("all").get("hosts")
    inven = json.loads(inven['dummy-test']).get("all").get("hosts")
    assert_equal(inven, expected)

@with_setup(setup_api)
def test_api_pinfile_get_inventory():
    pfile.up()
    inven = pfile.get_inventory()
    assert_equal.__self__.maxDiff = None
    expected = {u'dummy-test': '{"test": {"hosts": ["web-0"]}, "all": {"hosts": ["web-0", "web-1", "web-2", "test"], "vars": {"ansible_user": "root"}}, "example": {"hosts": ["web-1", "web-2", "test"]}, "_meta": {"hostvars": {"test": "test", "web-0": "web-0", "web-1": "web-1", "web-2": "web-2"}}}'}
    expected = json.loads(expected['dummy-test']).get("all").get("hosts")
    inven = json.loads(inven['dummy-test']).get("all").get("hosts")
    assert_equal(inven, expected)

@with_setup(setup_api)
def test_api_pinfile_validate():
    out = pfile.validate()
    expected = (0, {'dummy-test': {'layout': 'valid', 'topology': 'valid'}})
    assert_equal(expected, out)

@with_setup(setup_api)
def test_api_wksp_up():
    out = wksp.up()
    assert_equal(len(out), 2)

@with_setup(setup_api)
def test_api_wksp_destroy():
    out = wksp.destroy()
    assert_equal(len(out), 2)

@with_setup(setup_api)
def test_api_pinfile_up():
    out = pfile.up()
    assert_equal(len(out), 2)
    pass

@with_setup(setup_api)
def test_api_pinfile_destroy():
    out = pfile.destroy()
    assert_equal(len(out), 2)
    pass

def main():
    pass


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
import linchpin.FilterUtils.FilterUtils as filter_utils


class FilterModule(object):
    ''' A filter to format_output with omit '''
    def filters(self):
        return {
            'omit_filter': filter_utils.omit_filter
        }



#!/usr/bin/env python
import linchpin.FilterUtils.FilterUtils as filter_utils


class FilterModule(object):
    ''' A filter to prepare ssh args '''
    def filters(self):
        return {
            'prepare_ssh_args': filter_utils.prepare_ssh_args
        }



#!/usr/bin/env python

from __future__ import absolute_import
import linchpin.FilterUtils.FilterUtils as filter_utils


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'combine_hosts_names': filter_utils.combine_hosts_names
        }
